/*
 * Copyright (c) 2006-2007 Extreme! Lab, Indiana University. All rights reserved.
 *
 * This software is open source. See the bottom of this file for the license.
 *
 * $Id: GPELInvokeWindow.java,v 1.12 2007/06/22 19:02:38 sshirasu Exp $
 */
package edu.indiana.extreme.xbaya.gpel.gui;

import java.awt.event.ActionEvent;
import java.net.URI;
import java.net.URISyntaxException;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;
import javax.xml.namespace.QName;

import org.gpel.client.security.GpelUserX509Credential;
import org.ietf.jgss.GSSCredential;
import org.xmlpull.infoset.XmlElement;

import edu.indiana.extreme.xbaya.XBayaConfiguration;
import edu.indiana.extreme.xbaya.XBayaEngine;
import edu.indiana.extreme.xbaya.component.ComponentException;
import edu.indiana.extreme.xbaya.component.ws.WSComponentPort;
import edu.indiana.extreme.xbaya.gpel.GPELClient;
import edu.indiana.extreme.xbaya.gpel.GPELException;
import edu.indiana.extreme.xbaya.gpel.script.BPELScript;
import edu.indiana.extreme.xbaya.graph.GraphException;
import edu.indiana.extreme.xbaya.gui.ErrorMessages;
import edu.indiana.extreme.xbaya.gui.GridPanel;
import edu.indiana.extreme.xbaya.gui.XBayaDialog;
import edu.indiana.extreme.xbaya.gui.XBayaLabel;
import edu.indiana.extreme.xbaya.gui.XBayaTextArea;
import edu.indiana.extreme.xbaya.gui.XBayaTextComponent;
import edu.indiana.extreme.xbaya.gui.XBayaTextField;
import edu.indiana.extreme.xbaya.lead.LEADTypes;
import edu.indiana.extreme.xbaya.monitor.MonitorConfiguration;
import edu.indiana.extreme.xbaya.myproxy.MyProxyClient;
import edu.indiana.extreme.xbaya.myproxy.gui.MyProxyChecker;
import edu.indiana.extreme.xbaya.security.XBayaSecurity;
import edu.indiana.extreme.xbaya.util.XMLUtil;
import edu.indiana.extreme.xbaya.wf.Workflow;

/**
 * @author Satoshi Shirasuna
 */
public class GPELInvokeWindow {

    // private static final MLogger logger = MLogger.getLogger();

    private XBayaEngine engine;

    private Workflow workflow;

    private GPELInvoker invoker;

    private MyProxyChecker myProxyChecker;

    private XBayaDialog dialog;

    private GridPanel parameterPanel;

    private XBayaTextField topicTextField;

    private XBayaTextField resourceCatalogTextField;

    private XBayaTextField xRegistryTextField;

    private XBayaTextField gfacTextField;

    private XBayaTextField dscTextField;

    private List<XBayaTextComponent> parameterTextFields = new ArrayList<XBayaTextComponent>();

    private JButton redeployAndInvokeButton;

    private JButton deployNewAndInvokeButton;

    /**
     * Constructs a GPELInvokeWindow.
     * 
     * @param engine
     * 
     */
    public GPELInvokeWindow(XBayaEngine engine) {
        this.engine = engine;
        this.invoker = new GPELInvoker(engine);
        this.myProxyChecker = new MyProxyChecker(this.engine);
        initGUI();
    }

    /**
     * Shows the dialog.
     * 
     * @param compile
     *            If true, compile a workflow to create a workflow WSDL and a
     *            BPEL document. If false, use the ones included in the
     *            workflow.
     */
    public void show(boolean compile) {
        // Clean up the previous run. These cannot be in hide() because show()
        // might be called before hide() exits completly.
        this.parameterPanel.getContentPanel().removeAll();
        this.parameterTextFields.clear();

        GPELClient gpelClient = this.engine.getGPELClient();
        if (gpelClient.isSecure()) {
            // Check if the proxy is loaded.
            boolean loaded = this.myProxyChecker.loadIfNecessary();
            if (!loaded) {
                return;
            }
            // Creates a secure channel in gpel.
            MyProxyClient myProxyClient = this.engine.getMyProxyClient();
            GSSCredential proxy = myProxyClient.getProxy();
            GpelUserX509Credential credential = new GpelUserX509Credential(
                    proxy, XBayaSecurity.getTrustedCertificates());
            try {
                gpelClient.setGpelUserX509Credential(credential);
            } catch (GPELException e) {
                this.engine.getErrorWindow().error(ErrorMessages.GPEL_ERROR, e);
                return;
            }
        }

        this.workflow = this.engine.getWorkflow();

        MonitorConfiguration notifConfig = this.engine.getMonitor()
                .getConfiguration();
        if (notifConfig.getBrokerURL() == null) {
            this.engine.getErrorWindow().error(
                    ErrorMessages.BROKER_URL_NOT_SET_ERROR);
            return;
        }

        if (compile) {
            // Check if the workflow is valid before the user types in input
            // values.
            BPELScript bpel = new BPELScript(this.workflow);

            // Check if there is any errors in the workflow first.
            ArrayList<String> warnings = new ArrayList<String>();
            if (!bpel.validate(warnings)) {
                StringBuilder buf = new StringBuilder();
                for (String warning : warnings) {
                    buf.append("- ");
                    buf.append(warning);
                    buf.append("\n");
                }
                this.engine.getErrorWindow().warning(buf.toString());
                return;
            }

            try {
                // Generate a BPEL process.
                bpel.create();
                this.workflow.setGpelProcess(bpel.getGpelProcess());
                this.workflow.setWorkflowWSDL(bpel.getWorkflowWSDL()
                        .getWsdlDefinitions());
            } catch (GraphException e) {
                this.engine.getErrorWindow().error(
                        ErrorMessages.GRAPH_NOT_READY_ERROR, e);
                return;
            }
        } else {
            if (this.workflow.getWorkflowWSDL() == null) {
                this.engine.getErrorWindow().error(
                        ErrorMessages.WORKFLOW_WSDL_NOT_EXIST);
                return;
            }
        }

        // Enable/Disable redeploy button
        boolean enable = this.workflow.getGPELTemplateID() != null;
        this.redeployAndInvokeButton.setEnabled(enable);

        // Create a GUI without depending on the graph.
        List<WSComponentPort> inputs;
        try {
            inputs = this.workflow.getInputs();
        } catch (ComponentException e) {
            // This should not happen when we create WSDL here, but if we use
            // precompiled workflow, it might happen.
            this.engine.getErrorWindow().error(
                    ErrorMessages.WORKFLOW_WSDL_ERROR, e);
            return;
        }
        List<Double> columnWeights = new ArrayList<Double>();
        for (WSComponentPort input : inputs) {
            String id = input.getName();
            QName type = input.getType();
            JLabel paramLabel = new JLabel(id, SwingConstants.TRAILING);
            JLabel typeLabel = new JLabel(type.getLocalPart());
            XBayaTextComponent paramField;
            if (LEADTypes.isKnownType(type)) {
                paramField = new XBayaTextField();
                columnWeights.add(new Double(0));
            } else {
                paramField = new XBayaTextArea();
                columnWeights.add(new Double(1.0));
            }
            paramLabel.setLabelFor(paramField.getSwingComponent());

            // default value
            Object value = input.getDefaultValue();
            String valueString = null;
            if (value != null) {
                if (value instanceof XmlElement) {
                    XmlElement valueElement = (XmlElement) value;
                    valueString = XMLUtil.xmlElementToString(valueElement);
                } else {
                    // Only string comes here for now.
                    valueString = value.toString();
                }
            }

            if (valueString == null) {
                // show some sample URI to ease inputs.
                final String sampleURI = "gsiftp://rainier.extreme.indiana.edu//tmp/foo.txt";
                if (LEADTypes.isURIType(type)) {
                    valueString = sampleURI;
                } else if (LEADTypes.isURIArrayType(type)) {
                    StringBuffer buf = new StringBuffer();
                    for (int i = 0; i < 4; i++) {
                        buf.append(sampleURI).append(" ");
                    }
                    valueString = buf.toString();
                }
            }
            paramField.setText(valueString);

            this.parameterPanel.add(paramLabel);
            this.parameterPanel.add(typeLabel);
            this.parameterPanel.add(paramField);
            this.parameterTextFields.add(paramField);
        }
        List<Double> rowWeights = new ArrayList<Double>();
        rowWeights.add(new Double(0));
        rowWeights.add(new Double(0));
        rowWeights.add(new Double(1));
        this.parameterPanel.layout(columnWeights, rowWeights);

        XBayaConfiguration configuration = this.engine.getConfiguration();
        MonitorConfiguration monitorConfiguration = this.engine.getMonitor()
                .getConfiguration();

        // Topic
        String topic = monitorConfiguration.getTopic();
        this.topicTextField.setText(topic);

        // DSC URL
        this.dscTextField.setText(configuration.getDSCURL());

        // Resource Catalog
        this.resourceCatalogTextField.setText(configuration
                .getResourceCatalogURL());

        // XRegistry
        this.xRegistryTextField.setText(configuration.getXRegistryURL());

        // GFac URL
        this.gfacTextField.setText(configuration.getGFacURL());

        this.dialog.show();
    }

    /**
     * Hides the dialog.
     */
    public void hide() {
        this.dialog.hide();
    }

    private void initGUI() {
        this.parameterPanel = new GridPanel(true);

        this.topicTextField = new XBayaTextField();
        XBayaLabel topicLabel = new XBayaLabel("Notification Topic",
                this.topicTextField);

        this.resourceCatalogTextField = new XBayaTextField();
        XBayaLabel resourceCatalogLabel = new XBayaLabel(
                "Resource Catalog URL", this.resourceCatalogTextField);

        this.xRegistryTextField = new XBayaTextField();
        XBayaLabel xRegistryLabel = new XBayaLabel("XRegistry URL",
                this.xRegistryTextField);

        this.gfacTextField = new XBayaTextField();
        XBayaLabel gfacLabel = new XBayaLabel("GFac URL", this.gfacTextField);

        this.dscTextField = new XBayaTextField();
        XBayaLabel dscLabel = new XBayaLabel("DSC URL", this.dscTextField);

        GridPanel infoPanel = new GridPanel();
        infoPanel.add(topicLabel);
        infoPanel.add(this.topicTextField);
        infoPanel.add(dscLabel);
        infoPanel.add(this.dscTextField);
        infoPanel.add(resourceCatalogLabel);
        infoPanel.add(this.resourceCatalogTextField);
        infoPanel.add(xRegistryLabel);
        infoPanel.add(this.xRegistryTextField);
        infoPanel.add(gfacLabel);
        infoPanel.add(this.gfacTextField);
        infoPanel.layout(5, 2, GridPanel.WEIGHT_NONE, 1);

        GridPanel mainPanel = new GridPanel();
        mainPanel.add(this.parameterPanel);
        mainPanel.add(infoPanel);
        mainPanel.layout(2, 1, 0, 0);

        this.deployNewAndInvokeButton = new JButton("Deploy New and Invoke");
        this.deployNewAndInvokeButton.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                execute(true);
            }
        });

        this.redeployAndInvokeButton = new JButton("Redeploy and Invoke");
        this.redeployAndInvokeButton.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                execute(true);
            }
        });

        JButton cancelButton = new JButton("Cancel");
        cancelButton.addActionListener(new AbstractAction() {
            public void actionPerformed(ActionEvent e) {
                hide();
            }
        });

        JPanel buttonPanel = new JPanel();
        buttonPanel.add(this.deployNewAndInvokeButton);
        buttonPanel.add(this.redeployAndInvokeButton);
        buttonPanel.add(cancelButton);

        this.dialog = new XBayaDialog(this.engine, "Execute Workflow (BPEL)",
                mainPanel, buttonPanel);
    }

    private void execute(boolean redeploy) {

        // Get various values

        List<WSComponentPort> inputs;
        try {
            inputs = this.workflow.getInputs();
        } catch (ComponentException e) {
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
            hide();
            return;
        } catch (RuntimeException e) {
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
            hide();
            return;
        } catch (Error e) {
            this.engine.getErrorWindow().error(ErrorMessages.UNEXPECTED_ERROR,
                    e);
            hide();
            return;
        }

        int index = 0;
        for (XBayaTextComponent parameterTextField : this.parameterTextFields) {
            String valueString = parameterTextField.getText();
            WSComponentPort port = inputs.get(index++);
            // parse the value. parseValue pops up error if valueString is not
            // valid and return null.
            Object value = parseValue(port, valueString);
            if (value == null) {
                return;
            }
            port.setValue(value);
        }

        // Topic
        String topic = this.topicTextField.getText();
        if (topic.length() == 0) {
            this.engine.getErrorWindow().error(ErrorMessages.TOPIC_EMPTY_ERROR);
            return;
        }

        // DSC
        String dsc = this.dscTextField.getText();
        URI dscURL;
        if (dsc.length() == 0) {
            dscURL = null;
        } else {
            try {
                dscURL = new URI(dsc).parseServerAuthority();
            } catch (URISyntaxException e) {
                this.engine.getErrorWindow().error(ErrorMessages.DSC_URL_WRONG,
                        e);
                return;
            }
        }

        // Resource Catalog
        String resourceCatalog = this.resourceCatalogTextField.getText();
        URI resourceCatalogURL;
        if (resourceCatalog.length() == 0) {
            resourceCatalogURL = null;
        } else {
            try {
                resourceCatalogURL = new URI(resourceCatalog)
                        .parseServerAuthority();
            } catch (URISyntaxException e) {
                this.engine.getErrorWindow().error(
                        ErrorMessages.RESOURCE_CATALOG_URL_WRONG, e);
                return;
            }
        }

        // XRegistry
        String xRegistry = this.xRegistryTextField.getText();
        URI xRegistryURL;
        if (xRegistry.length() == 0) {
            xRegistryURL = null;
        } else {
            try {
                xRegistryURL = new URI(xRegistry).parseServerAuthority();
            } catch (URISyntaxException e) {
                this.engine.getErrorWindow().error(
                        ErrorMessages.XREGISTRY_URL_WRONG, e);
                return;
            }
        }

        // GFac
        String gfac = this.gfacTextField.getText();
        URI gfacURL;
        if (gfac.length() == 0) {
            gfacURL = null;
        } else {
            try {
                gfacURL = new URI(gfac).parseServerAuthority();
            } catch (URISyntaxException e) {
                this.engine.getErrorWindow().error(
                        ErrorMessages.GFAC_URL_WRONG, e);
                return;
            }
        }

        // Set to the config so that they will be reused.
        MonitorConfiguration monitorConfig = this.engine.getMonitor()
                .getConfiguration();
        monitorConfig.setTopic(topic);
        XBayaConfiguration config = this.engine.getConfiguration();
        config.setDSCURL(dscURL);
        config.setResourceCatalogURL(resourceCatalogURL);
        config.setXRegistryURL(xRegistryURL);
        config.setGFacURL(gfacURL);

        this.invoker.invoke(this.workflow, inputs, redeploy);

        hide();
    }

    /**
     * @param input
     * @param valueString
     * @return The parsed value
     */
    private Object parseValue(WSComponentPort input, String valueString) {
        String name = input.getName();
        if (false) {
            // Some user wants to pass empty strings, so this check is disabled.
            if (valueString.length() == 0) {
                this.engine.getErrorWindow().error(
                        "Input parameter, " + name + ", cannot be empty");
                return null;
            }
        }
        QName type = input.getType();
        Object value;
        if (LEADTypes.isKnownType(type)) {
            // TODO check the type.
            value = valueString;
        } else {
            try {
                value = XMLUtil.stringToXmlElement3(valueString);
            } catch (RuntimeException e) {
                this.engine.getErrorWindow().error(
                        "Input parameter, " + name + ", is not valid XML", e);
                return null;
            }
        }
        return value;
    }
}

/*
 * Indiana University Extreme! Lab Software License, Version 1.2
 * 
 * Copyright (c) 2006-2007 The Trustees of Indiana University. All rights
 * reserved.
 * 
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 * 
 * 1) All redistributions of source code must retain the above copyright notice,
 * the list of authors in the original source code, this list of conditions and
 * the disclaimer listed in this license;
 * 
 * 2) All redistributions in binary form must reproduce the above copyright
 * notice, this list of conditions and the disclaimer listed in this license in
 * the documentation and/or other materials provided with the distribution;
 * 
 * 3) Any documentation included with all redistributions must include the
 * following acknowledgement:
 * 
 * "This product includes software developed by the Indiana University Extreme!
 * Lab. For further information please visit http://www.extreme.indiana.edu/"
 * 
 * Alternatively, this acknowledgment may appear in the software itself, and
 * wherever such third-party acknowledgments normally appear.
 * 
 * 4) The name "Indiana University" or "Indiana University Extreme! Lab" shall
 * not be used to endorse or promote products derived from this software without
 * prior written permission from Indiana University. For written permission,
 * please contact http://www.extreme.indiana.edu/.
 * 
 * 5) Products derived from this software may not use "Indiana University" name
 * nor may "Indiana University" appear in their name, without prior written
 * permission of the Indiana University.
 * 
 * Indiana University provides no reassurances that the source code provided
 * does not infringe the patent or any other intellectual property rights of any
 * other entity. Indiana University disclaims any liability to any recipient for
 * claims brought by any other entity based on infringement of intellectual
 * property rights or otherwise.
 * 
 * LICENSEE UNDERSTANDS THAT SOFTWARE IS PROVIDED "AS IS" FOR WHICH NO
 * WARRANTIES AS TO CAPABILITIES OR ACCURACY ARE MADE. INDIANA UNIVERSITY GIVES
 * NO WARRANTIES AND MAKES NO REPRESENTATION THAT SOFTWARE IS FREE OF
 * INFRINGEMENT OF THIRD PARTY PATENT, COPYRIGHT, OR OTHER PROPRIETARY RIGHTS.
 * INDIANA UNIVERSITY MAKES NO WARRANTIES THAT SOFTWARE IS FREE FROM "BUGS",
 * "VIRUSES", "TROJAN HORSES", "TRAP DOORS", "WORMS", OR OTHER HARMFUL CODE.
 * LICENSEE ASSUMES THE ENTIRE RISK AS TO THE PERFORMANCE OF SOFTWARE AND/OR
 * ASSOCIATED MATERIALS, AND TO THE PERFORMANCE AND VALIDITY OF INFORMATION
 * GENERATED USING SOFTWARE.
 */

